@@ -1,6 +1,7 @@
 #ifndef __rockdodger_INTRO_H__
 #define __rockdodger_INTRO_H__
 #include "scroller.h"
+#include "u-iff.h"
 
 #define NUM_COPPERBARS 6
 
@@ -15,16 +16,17 @@ struct rockintro {
 /*! \brief Initialise the intro.
  *
  * \param target Surface to be painted later on (for witdth, etc.)
+ * \param iff context for loading parameters for the intro
  * \return NULL on error or pointer to allocated rockintro structure
  */
-struct rockintro *init_intro(SDL_Surface *target);
+struct rockintro *init_intro(SDL_Surface *target, uiff_ctx_t *iff);
 
 
 /*! \brief Do the intro thingy.
  *
  * \param intro Pointer to the intro structure with the current state 
  * \param target SDL_Surface to draw on.
- * \param movementrate Relative speed to 50 fps, see rockdodger_globals.h.
+ * \param movementrate Relative speed to 50 fps, see globals.h.
  */
 void update_and_draw_intro(struct rockintro *intro, SDL_Surface *target, float movementrate);
 
@@ -43,7 +45,8 @@ void shutdown_intro(struct rockintro *intro);
  * 
  * \param force if nonzero then always play the intro
  * \param oss_sound_flag if nonzero then play a sound
+ * \param iff context for loading parameters for the intro
  */
-void play_intro(int force, int oss_sound_flag);
+void play_intro(int force, int oss_sound_flag, uiff_ctx_t iff);
 
 #endif
