@@ -29,7 +29,7 @@
 #include <math.h>
 
 #include "config.h"
-#include "rockdodger_globals.h"
+#include "globals.h"
 #include "SFont.h"
 #include "sound.h"
 #include "random_gen.h"
@@ -50,10 +50,14 @@
 #include "ship.h"
 #include "datafun.h"
 #include "signal_handling.h"
+#include "sekrit_lab.h"
+#include "yellifish.h"
+#include "infoscreen.h"
 
 // constants
 #define M 255
 #define BIG_FONT_FILE "20P_Betadance.png"
+#define ROCKIFF "iff.configdata.rock" //!< rockdodger iff file
 
 // macros
 #define COND_ERROR(a) if ((a)) {initerror=strdup(SDL_GetError());return 1;}
@@ -68,11 +72,16 @@ SDL_Surface *surf_font_big;	// The big font
 SDL_Surface *surf_gauge_shield[2];	// The shield gauge 0=on,1=off
 SDL_Surface *surf_gauge_laser[2];	// The laser gauge 0=on,1=off
 
-Scroller_t *highscore_scroller = NULL; // Scroller for highscore
+Scroller_t *highscore_scroller = NULL; //!< Scroller for highscore
+
+yellifishsubsystem_t *yellifishs; //!< array of yellifishs and more
+yellifish_t yellifish_for_infoscreen;
+SDL_Surface *mood_icon; //!< mood item icon for the infoscreen
 
 // Other global variables
 char topline[128];
 char *initerror = "";
+char *rockiff_name = NULL; //!< filename of rockdodger binary (iff) file with parameters/manipulation/configuration contents
 
 float rockrate, rockspeed;
 
@@ -106,7 +115,10 @@ char *sequence[] = {
   "...the very dangerous Blubats!",
   "Watch out! Blubats may leave some droppings!",
   "Blubat droppings should not be inhaled!",
-  "",
+  "Further on, you will meet the Yellifish,",
+  "devious creatures they are... Beware!",
+   "Cunning predators hunting you down!",
+   "",
   "Code: RPK, Paul Holt",
   "GfX: RPK, Paul Holt",
   "Music: Jack Beatmaster, zake, Strobo, Roz",
@@ -140,9 +152,11 @@ int restart_game() {
   const char *txts[] = {
     "Congratulations! You have a high score.",
     "We have a winner!",
+    "Wot a hero!",
     "A place in the hall of fame has been reserved for you!",
     "Amazing performance! You earned a high score.",
-    "Unbelievable, do you do anything besides playing this game?"
+    "Unbelievable, do you do anything besides playing this game?",
+    "Oh, come on... Admit it... You cheated, did you not?"
   };
   const int num_of_txts = sizeof(txts) / sizeof(const char *);
   char buf[160];
@@ -173,6 +187,7 @@ int restart_game() {
 
 void drawlaser() {
   int i, xc, hitrock, hitgreeb;
+  yellifish_t *yptr;
 
   if(laserlevel < 0)
     return;
@@ -182,7 +197,6 @@ void drawlaser() {
     laserlevel = -W / 2;
     return;
   }
-
   hitrock = -1;
   hitgreeb = -1;
   xc = xsize;
@@ -219,14 +233,16 @@ void drawlaser() {
 	hitgreeb = i;
       }
     }
-
   }
 
   if(hitrock >= 0)
     heat_rock_up(hitrock, movementrate);
   if(hitgreeb >= 0)
     kill_greeb(hitgreeb);
-
+  if((yptr = check_yellifish_hit(yellifishs, xship + 32, xc, yship + 12))) {
+    inc_score(xship + 32, yship + 12, YELLIFISH_KILL_POINTS);
+    kill_yellifish(yptr);
+  }
   if((hitgreeb = check_blubat_hit((int) yship + 12, xc)) >= 0) {
     kill_blubat(hitgreeb);
   }
@@ -256,6 +272,8 @@ SDL_Surface *set_video_mode() {
 int init() {
   int i;
 
+  // Make sure that the name_input_buf is null terminated.
+  *name_input_buf = '\0';
   read_high_score_table();
   data_dir = get_data_dir();
   if(oss_sound_flag) {
@@ -278,7 +296,8 @@ int init() {
   // Attempt to get the required video size
   set_video_mode();
   NULL_ERROR(surf_screen);
-  NULL_GURU(iff_ctx = get_iff_rock(load_file("iff.configdata.rock")), GM_FLAGS_RECOVERY | GM_FLAGS_ABORTIFY | GM_FLAGS_CHOICE | GM_SS_BootStrap | GM_GE_OpenLib | GURU_SEC_main, &get_iff_rock);
+  NULL_GURU(iff_ctx = get_iff_rock(rockiff_name != NULL ? rockiff_name : load_file(ROCKIFF)), GM_FLAGS_DEADEND | GM_FLAGS_ABORTIFY | GM_FLAGS_CHOICE | GM_SS_BootStrap | GM_GE_OpenLib | GURU_SEC_main, &get_iff_rock);
+  edition_name = load_edition_name(*iff_ctx);
   //TODO: Add a splash screen...
   SDL_FillRect(surf_screen, NULL, SDL_MapRGB(surf_screen->format, 0x2e, 0x2e, 0x2e));
   SDL_Flip(surf_screen);
@@ -320,12 +339,13 @@ int init() {
 			      // Got that?
 			      );
 #ifdef DEBUG
-  printf("heatcolor[%03x] = { %02x, %02x, %02x }\n", i,
+  printf("heatcolor[%04x] = { %02x, %02x, %02x }\t", i,
 			      (i < W) ? (i * M / W) : (M),
 			      (i < W) ? 0 : (i <
 					     2 * W) ? ((i - W) * M / W) : M,
 			      (i < 2 * W) ? 0 : ((i - W) * M / W)
 	 ); //No, not really...
+  if(i == W * 3 - 1) putchar('\n');
 #endif
   }
   // Load global font
@@ -352,12 +372,15 @@ int init() {
 
   NULL_GURU(init_ship(), 0x40000000 | GM_GE_ProcCreate, 0x53484950);
   init_engine_dots();
+  NULL_GURU(init_infoscreen(*iff_ctx), GM_SS_Misc | GM_GE_OpenDev | GURU_SEC_infoscreen, &init_infoscreen);
   NULL_ERROR(init_blubats(*iff_ctx));
   NULL_ERROR(init_powerup());
-  NULL_ERROR(init_mood_item());
+  NULL_ERROR(mood_icon = init_mood_item());
   // Load all our lovely rocks
   NULL_ERROR(init_rocks());
   NULL_ERROR(init_greeblies());
+  NULL_ERROR((yellifishs = init_yellifish_subsystem(*iff_ctx)));
+  init_yellifish(yellifishs, &yellifish_for_infoscreen, 0, 0);
   NULL_GURU(init_sparkles(*iff_ctx), GM_SS_Graphics | GM_GE_OpenLib | GURU_SEC_sparkles, &init_sparkles);
   // Remove the mouse cursor
 #ifdef SDL_DISABLE
@@ -403,7 +426,9 @@ void showgauge(int x, SDL_Surface * surf[2], float fraction) {
 void display_version_n_text() {
   char buf[120];
   char *text;
-  sprintf(buf, "Version %s - The %s edition  ( %s )", VERSION, current_edition == EDITION_greeblies ? "GREEBLIES" : "BLUBAT", COMPILEDATE);
+
+  assert(edition_name);
+  sprintf(buf, "Version %s - The %s edition  ( %s )", VERSION, edition_name, COMPILEDATE);
   int x = (xsize - SFont_wide(buf)) / 2 + sinf(fadetimer / 4.5) * 10;
   PutString(surf_screen, x, ysize - 50 + sinf(fadetimer / 2) * 5, buf);
 
@@ -424,9 +449,9 @@ void display_highscores() {
   for(i = 0; i < MAXIMUM_HIGH_SCORE_ENTRIES; ++i) {
     sprintf(s, "#%1d", i + 1);
     PutString(surf_screen, 150, 50 + (i + 2) * font_height, s);
-    sprintf(s, "%4ld", high[current_edition][i].score);
+    sprintf(s, "%4ld", high[i].score);
     PutString(surf_screen, 200, 50 + (i + 2) * font_height, s);
-    sprintf(s, "%3s", high[current_edition][i].name);
+    sprintf(s, "%3s", high[i].name);
     PutString(surf_screen, 330, 50 + (i + 2) * font_height, s);
   }
 }
@@ -444,9 +469,9 @@ void enter_highscore(void) {
     if(SFont_Input
        (surf_screen, 330, 50 + (scorerank + 2) * font_height, 300, name_input_buf)) {
       // Insert new high score and name into high score table
-      high[current_edition][scorerank].score = score;
-      high[current_edition][scorerank].name = strdup(name_input_buf);
-      high[current_edition][scorerank].allocated = 1;
+      high[scorerank].score = score;
+      high[scorerank].name = strdup(name_input_buf);
+      high[scorerank].allocated = 1;
       scorerank = -1;		//we are unworthy again
 
       // Set the global name string to "", ready for the next winner
@@ -517,6 +542,42 @@ void draw_gameover(void) {
 }
 
 
+void yellifish_displayer(SDL_Surface *target, SDL_Rect *dest) {
+  yellifish_for_infoscreen.posx = dest->x + yellifishs->headoffsetx;
+  yellifish_for_infoscreen.posy = dest->y + yellifishs->headoffsety;
+  display_yellifish(yellifishs, &yellifish_for_infoscreen, target);
+}
+
+
+void display_infoscreen() {
+  struct Entry infotxt[] = {
+#ifndef NDEBUG
+    { "DEBUG", 640, 400, {INFOICON, {surf_small_ship}}, -1 },
+#endif
+    { "The good", 0, 0, {INFONONE}, 0 },
+    { "", 0, 0, {INFONONE}, 0 },
+    { "Laser powerup", 0, 0, {INFOICON, {surf_powerups[POWERUP_LASER]->surfaces[0]}}, 0 },
+    { "shield powerup", 0, 0, {INFOICON, {surf_powerups[POWERUP_SHIELD]->surfaces[0]}}, 0 },
+    { "Extra life", 0, 0, {INFOICON, {surf_powerups[POWERUP_LIFE]->surfaces[0]}}, 0 },
+    /* ------------------------------------------------------------ */
+    { "The bad", 0, 0, {INFONONE}, 1 },
+    { "", 0, 0, {INFONONE}, 1 },
+    { "Rocks", 0, 0, {INFOICON, {get_rock_surface(0, 0)}}, 1 },
+    { "Greeblies (100 pts)", 0, 0, {INFOICON, {surf_greeblies->surfaces[0]}}, 1 },
+    { "Blubats (125 pts)", 0, 0, {INFOICON, {surf_blubats->surfaces[0]}}, 1 },
+    { "Yellifish (175 pts)", 0, 0, {INFOFUNP, {.icondisplayfun = &yellifish_displayer}}, 1},
+    /* ------------------------------------------------------------ */
+    { "The other", 0, 0, {INFONONE}, 2 },
+    { "", 0, 0, {INFONONE}, 2 },
+    { "Your ship", 0, 0, {INFOICON, {surf_small_ship}}, 2 },
+    { "Mood items (harmless)", 0, 0, {INFOICON, {mood_icon}}, 2 },
+    { NULL }
+  };
+
+  display_infotext(infotxt, surf_screen);
+}
+
+
 /*! \brief Draw all
  *
  * Draw all the objects on the standard surface.
@@ -547,6 +608,8 @@ int draw() {
   display_rocks(surf_screen);
   // Draw all blubats
   display_blubats(surf_screen);
+  // Draw the yellifish
+  display_yellifishs(yellifishs, surf_screen);
 
   switch (state) {
   case GAME_OVER:
@@ -566,6 +629,10 @@ int draw() {
     display_highscores();
     display_version_n_text();
     break;
+  case INFO_SCREEN:
+    display_infoscreen();
+    display_version_n_text();
+    break;
   case GAME_PAUSED:
     {
       SDL_Rect dest;
@@ -580,7 +647,7 @@ int draw() {
   case GAME_PLAY:
     break;
   case DEMO:
-#if defined(DEMO_ENABLED) && defined(DEBUG)
+#if defined(DEBUG)
     {
       int i;
       for(i = 0; i < rockfree_space_num; ++i) {
@@ -630,8 +697,9 @@ int draw() {
 	  case POWERUP_LIFE:
 	    nships += 1; //Increase number of ships.
 	    break;
+	  case POWERUP_MAXPOWERUP:
 	  case POWERUP_NONE:
-	    puts("Illegal powerup (NONE?)!");
+	    puts("Illegal powerup!");
 	    play_sound(4);
 	    break;
 	  }
@@ -680,8 +748,8 @@ void crash_boom_bang() {
     // Play the explosion sound
     play_sound(0);
   }
-  way_of_the_exploding_ship();
   if(state != DEMO) {
+    way_of_the_exploding_ship();
     if(--nships <= 0) {
       state = GAME_OVER;
       state_timeout = 200.0;
@@ -694,6 +762,8 @@ void crash_boom_bang() {
 			       will not black the background.*/
       //Remove blubats
       deactivate_all_blubats();
+      //Remove yellifish
+      deactivate_all_yellifish(yellifishs);
     } else {
       state = DEAD_PAUSE;
       state_timeout = 120.0;
@@ -726,7 +796,9 @@ void ready_for_takeoff() {
   for(i = 0; i < MAX_ROCKS; i++)
     rock[i].active = 0;
   deactivate_greeblies();
-  rockspeed = 5.0 * xsize / 640;
+  deactivate_all_blubats();
+  deactivate_all_yellifish(yellifishs);
+  rockspeed = 5.0 * xsize / 640.0;
   rockrate = 0;
   countdown = 1.111;
   level = 1;
@@ -753,7 +825,7 @@ int gameloop() {
   while(state != QUIT_GAME && state != RESTART_GAME) {
     update_movement_rate();
     keystate = input_subsystem(&inputstate);
-    handle_state_timeout();
+    handle_state_timeout(infoscreen_enabled);
 
     if(state != GAME_PAUSED) {
       rockspeed = 4.885 * xsize / 640.0 + sqrtf(floorf(level)) / 5.0;
@@ -835,9 +907,10 @@ int gameloop() {
       // Increase difficulty
       level += movementrate / 250;
       //Activate a blubat if necessary
-      if((current_edition == EDITION_blubat) && (rnd() < (level - 8) / 777 && ((int)level) % 5 != 0)) { //Every 5th level no blubats
+      if(rnd() < (level - 5) / 797.141347 && ((int)level) % 5 != 0) { //Every 5th level no blubats
 	activate_one_blubat();
       }
+      maybe_activate_yellifish(yellifishs, (int)level, xsize + 34, -24);
       //Update greeblies
       if(rnd() < .02 * level * movementrate) {
 	activate_one_greeblie();
@@ -851,8 +924,11 @@ int gameloop() {
       move_all_greeblies();
       //Update blubats
       move_all_blubats();
+      //Update yellifishs
+      update_yellifishs(yellifishs, xship, yship - 87.8228228);
       update_sparkles(surf_screen);
       break;
+    case INFO_SCREEN:
     case GAME_OVER:
     case HIGH_SCORE_DISPLAY:
     case HIGH_SCORE_ENTRY:
@@ -865,6 +941,7 @@ int gameloop() {
       move_all_greeblies();
       //Update blubats
       move_all_blubats();
+      update_yellifishs(yellifishs, xsize, 0);
       update_sparkles(surf_screen);
       break;
     case SETUP:
@@ -915,12 +992,6 @@ int gameloop() {
       if(inputstate.inputstate[LASER]) {
 	laser = 1;
       }
-      if(inputstate.inputstate[FAST]) {
-	fast = 1;
-      }
-      if(inputstate.inputstate[NOFAST]) {
-	fast = 0;
-      }
       shieldsup = inputstate.inputstate[SHIELD];
     }
     //Check for screen shot.
@@ -932,137 +1003,25 @@ int gameloop() {
       ret = SDL_SaveBMP(surf_screen, buf);
       fprintf(stderr, "Screenshot '%s' saved %s.\n", buf, ret == 0 ? "successfully" : "unsuccessfully(!)");
     }
-
-    // DEBUG mode to slow down the action, and see if this game is playable on a 486
-    if(fast)
-      SDL_Delay(100);
   }
   return state == RESTART_GAME ? 1 : 0;
 }
 
 
-/*! \brief Perform one of the tests.
- *
- * For debugging only!
- *
- * \return 0 = no valid test found, otherwise 1
- */
-int perform_a_sekrit_test(char *arg) {
-  SDL_Event event;
-  int running = 1;
-
-  printf("Looking for test '%s'...\n", arg);
-  movementrate = 1.0;
-  if(strcmp(arg, "guru") == 0) {
-    SDL_FillRect(surf_screen, NULL, SDL_MapRGB(surf_screen->format, 0, 0x55, 0xAA));
-    printf("guru_display() = %d\n", guru_display_gp(surf_screen, GM_FLAGS_AUTOTIMEOUT | 2, 3, (void*)0xDEADBEA1, "Greeblies at work..."));
-    printf("guru_display() = %d\n", guru_display_gp(surf_screen, GM_FLAGS_AUTOTIMEOUT| 1 | GM_FLAGS_CHOICE, 3, (void*)0xDEADBEA1, "Alert! This needs still a lot of work!"));
-    printf("guru_display() = %d\n", guru_display_gp(surf_screen, GM_FLAGS_AUTOTIMEOUT | 0, 4, (void*)0xAA55AA55, "Normal..."));
-    printf("guru_meditation() = %d\n", guru_meditation(GM_FLAGS_DEADEND | GM_FLAGS_EXIT_VIA_TERM, 0x48454c50, (void*)0x68656c70));
-    return 1;
-  } else if(strcmp(arg, "laserpointer") == 0) {
-    float positions[10];
-    float speeds[10];
-    int i;
-    int ysize2 = ysize / 2 - 3;
-
-    for(i = 0; i < 10; ++i) {
-      speeds[i] = rnd() / 100;
-      positions[i] = 0;
-    }
-    
-    while(running) {
-      if(SDL_PollEvent(&event) != 0 && event.type == SDL_MOUSEBUTTONDOWN) running = 0;
-      SDL_FillRect(surf_screen, NULL, SDL_MapRGB(surf_screen->format, 0, 0, 0x13));
-      for(i = 0; i < 10; ++i) {
-	draw_random_dots(surf_screen, sinf(positions[i]) * ysize2 + ysize2 + 1, 0, xsize);
-	positions[i] += speeds[i];
-      }
-      SDL_Flip(surf_screen);
-      SDL_Delay(60);
-    }
-    return 1;
-  } else if(strcmp(arg, "sparkles") ==0) {
-    int i, j;
-    float x;
-    RD_VIDEO_TYPE *rawpixel;
-
-    while(running) {
-      if(SDL_PollEvent(&event) != 0 && event.type == SDL_MOUSEBUTTONDOWN) running = 0;
-      SDL_FillRect(surf_screen, NULL, SDL_MapRGB(surf_screen->format, 0, 0, 0x13));
-      SDL_LockSurface(surf_screen);
-      rawpixel = (RD_VIDEO_TYPE *) surf_screen->pixels;
-      for(i = 0; i < xsize; ++i) {
-	x = i / (xsize - 1.0);
-	for(j = 10; j < 50; ++j) {
-	  rawpixel[surf_screen->pitch / sizeof(RD_VIDEO_TYPE) * j + i] = (RD_VIDEO_TYPE)get_life_colour(x, &hot_colours, surf_screen);
-	}
-	for(j = 110; j < 150; ++j) {
-	  rawpixel[surf_screen->pitch / sizeof(RD_VIDEO_TYPE) * j + i] = (RD_VIDEO_TYPE)get_life_colour(x, &cool_colours, surf_screen);
-	}
-      }
-      SDL_UnlockSurface(surf_screen);
-      SDL_Flip(surf_screen);
-      SDL_Delay(60);
-    }
-    return 1;
-  } else if(strcmp(arg, "key_lab") ==0) {
-    unsigned int event_nr = 0, k;
-    unsigned int row = 0;
-    char buf[30][128];
-    SDL_Rect rect = { 0, 0, surf_screen->w, 20 };
-
-    restart_game();
-    assert(surf_screen != NULL);
-    assert(surf_green_block != NULL);
-    memset(buf, 0, sizeof(buf));
-    while(running) {
-      while(SDL_PollEvent(&event) == 1) {
-	row = event_nr % 30;
-	switch(event.type) {
-	case SDL_QUIT:
-	  running = 0;
-	  break;
-	case SDL_MOUSEBUTTONDOWN:
-	case SDL_MOUSEBUTTONUP:
-	  sprintf(buf[row], "button=%d state=%d x=$%04hX y=$%04hX", event.button.button, event.button.state, event.button.x, event.button.y);
-	  break;
-	case SDL_KEYDOWN:
-	case SDL_KEYUP:
-	  sprintf(buf[row], "state=%d sym=$%04X", event.key.state, event.key.keysym.sym);
-	  break;
-	case SDL_MOUSEMOTION:
-	  //Ignore this one...
-	  --event_nr;
-	  break;
-	default:
-	  sprintf(buf[row], "event_nr = $%x, event.type = $%x", event_nr, (unsigned int)event.type);
-	}
-	++event_nr;
-      }
-      SDL_FillRect(surf_screen, NULL, SDL_MapRGB(surf_screen->format, 0xf, 0, 0xf));
-      SDL_SetAlpha(surf_green_block, SDL_SRCALPHA, (int) (129 + 66 * sinf(fadetimer)));
-      rect.y = row * 20;
-      SDL_BlitSurface(surf_green_block, NULL, surf_screen, &rect);
-      for(k = 0; k < 30; ++k) {
-	PutString(surf_screen, 12, k * 20, buf[k]);
-      }
-      SDL_Flip(surf_screen);
-      SDL_Delay(50);
-      fadetimer += .192;
-    }
-    return 1;
-  }
-  puts("Not found!");
-  return 0;
-}
-
 void shutdown_subsystems() {
+  int i;
+
   shutdown_greeblies();
   shutdown_blubats();
   shutdown_powerups();
-  fclose(iff_ctx->f);
-  free(iff_ctx);
+  shutdown_yellifish_subsystem(yellifishs);
+  destroy_mood_item();
+  i = uiff_close(iff_ctx);
+  assert(printf("close IFF context returned $%08"PRIx32"\n", i));
+  if(i) fprintf(stderr, "Error on closing IFF context!\n");
+  //We can free this safely as rockiff_name is either a result of strdup or NULL.
+  free(rockiff_name);
+  free(edition_name);
 }
 
 
@@ -1073,8 +1032,11 @@ int main(int argc, char **argv) {
   assert(RD_VIDEO_BPP == 8 * sizeof(RD_VIDEO_TYPE));
   oss_sound_flag = 1;
   load_setup();
-  while((x = getopt(argc, argv, "Ifwskhx:y:")) >= 0)
+  while((x = getopt(argc, argv, "P:Ifwskhx:y:")) >= 0)
     switch (x) {
+    case 'P':
+      rockiff_name = strdup(optarg);
+      break;
     case 'I':
       force_intro = 1;
       break;
@@ -1121,7 +1083,7 @@ int main(int argc, char **argv) {
     //A non option argument was given!
     printf("Testing for %lu ticks.\n", (unsigned long int)SDL_GetTicks());
   } else {
-    play_intro(force_intro, oss_sound_flag);
+    play_intro(force_intro, oss_sound_flag, *iff_ctx);
     //Sometimes the game slips directly from intro to gameplay if space was pressed.
     last_ticks = SDL_GetTicks();
     temporary_disable_key_input();
