@@ -4,10 +4,14 @@
 
 /*! \brief Initialise mood items
  *
- * \return NULL on error, else something else
+ * \return NULL on error, else return mood-item icon
  */
-void *init_mood_item(void);
+SDL_Surface *init_mood_item(void);
 
 void draw_mood_item(SDL_Surface *target);
 
+/*! \brief destroy mood item subsystem
+ */
+void destroy_mood_item(void);
+
 #endif
