@@ -34,3 +34,7 @@ void init_sprite(struct sprite *sprite, SDL_Surface ** surfaces,
 	 sprite, sprite->start_tick, sprite->speed_divisor, surface_count);
 #endif
 }
+
+void init_sprite2(struct sprite *sprite, sdl_surfaces_t *surfaces, Uint16 delay, Uint32 now) {
+  init_sprite(sprite, surfaces->surfaces, surfaces->num_surfaces, delay, now);
+}
